@@ -1,47 +1,28 @@
 # arpakit
-import datetime as dt
-from typing import Any, Optional, get_type_hints, get_origin, Union, Annotated, get_args
+from typing import Any, get_type_hints, get_origin, Union, Annotated, get_args
 
-from pydantic import BaseModel, Field, ConfigDict
+from pydantic import BaseModel, ConfigDict
 from sqlalchemy import inspect
-from sqlalchemy.orm import ColumnProperty
-from sqlalchemy.sql.sqltypes import (
-    Boolean, Integer, BigInteger, SmallInteger,
-    String, Text, Unicode, UnicodeText,
-    DateTime, Date, Time,
-    Float, Numeric, DECIMAL, LargeBinary, JSON
-)
+from sqlalchemy.orm import ColumnProperty, Mapped
 
 _ARPAKIT_LIB_MODULE_VERSION = "3.0"
 
-_SQLA_TYPE_MAP = {
-    Boolean: bool,
-    Integer: int,
-    BigInteger: int,
-    SmallInteger: int,
-    Float: float,
-    Numeric: float,
-    DECIMAL: float,
-    String: str,
-    Unicode: str,
-    Text: str,
-    UnicodeText: str,
-    LargeBinary: bytes,
-    JSON: dict | list,
-    DateTime: dt.datetime,
-    Date: dt.date,
-    Time: dt.time,
-}
 
+def _define_sqlalchemy_column_mapped_type(type_: Any) -> Any:
+    """
+    Возвращает тип колонки ИСКЛЮЧИТЕЛЬНО из аннотации поля.
+    Разворачивает оболочки:
+      - Annotated[T, ...] -> T
+      - Mapped[T] -> T
+    Если аннотации нет — возвращает Any.
+    """
 
-def _python_type_from_col(col) -> type | str:
-    try:
-        return col.type.python_type
-    except Exception:
-        for sa_t, py_t in _SQLA_TYPE_MAP.items():
-            if isinstance(col.type, sa_t):
-                return py_t
-        return Any
+    origin = get_origin(type_)
+
+    if origin is Mapped:
+        return get_args(type_)[0] if get_args(type_) else Any
+
+    return type_
 
 
 def _get_property_name_to_type_from_model_class(
@@ -64,8 +45,9 @@ def _get_property_name_to_type_from_model_class(
                 ret_type = hints.get("return", Any)
             except Exception:
                 if skip_property_if_cannot_define_type:
-                    continue
-                ret_type = Any
+                    ret_type = Any
+                else:
+                    raise
             if exclude_property_names:
                 if property_name in exclude_property_names:
                     continue
@@ -108,12 +90,22 @@ def _type_matches(*, type_: Any, allowed_types: list[type]) -> bool:
     return False
 
 
+def _get_sqlalchemy_mapped_types(sqlalchemy_model):
+    result = {}
+    for cls in reversed(sqlalchemy_model.__mro__):
+        annotations = getattr(cls, "__annotations__", {})
+        for field, annotation in annotations.items():
+            if get_origin(annotation) is Mapped:
+                result[field] = annotation
+    return result
+
+
 def pydantic_schema_from_sqlalchemy_model(
         sqlalchemy_model: type,
         *,
         model_name: str | None = None,
         base_model: type[BaseModel] = BaseModel,
-        include_column_defaults: bool = False,
+        include_columns: bool = True,
         exclude_column_names: list[str] | None = None,
         include_properties: bool = False,
         include_property_names: list[str] | None = None,
@@ -122,7 +114,9 @@ def pydantic_schema_from_sqlalchemy_model(
         exclude_property_types: list[type] | None = None,
         filter_property_prefixes: list[str] | None = None,
         remove_property_prefixes: list[str] | None = None,
-        skip_property_if_cannot_define_type: bool = True
+        skip_property_if_cannot_define_type: bool = True,
+        skip_property_name_if_exists: bool = True,
+        model_name_suffix: str = "Schema"
 ) -> type[BaseModel]:
     """
     Генерирует Pydantic-модель из колонок SQLAlchemy-модели и (опционально) из @property.
@@ -135,7 +129,7 @@ def pydantic_schema_from_sqlalchemy_model(
     - exclude_property_names: blacklist имён свойств (исключаются после whitelist'а).
     """
     mapper = inspect(sqlalchemy_model).mapper
-    model_name = model_name or f"{sqlalchemy_model.__name__}Schema"
+    model_name = model_name or f"{sqlalchemy_model.__name__}{model_name_suffix}"
 
     annotations: dict[str, Any] = {}
     attrs: dict[str, Any] = {}
@@ -144,34 +138,19 @@ def pydantic_schema_from_sqlalchemy_model(
     include_property_names = set(include_property_names or [])
     exclude_property_names = set(exclude_property_names or [])
     filter_property_prefixes = set(filter_property_prefixes or [])
-    remove_property_prefixes = set(remove_property_prefixes or [])
+    remove_property_prefixes = list(set(remove_property_prefixes or []))
+    remove_property_prefixes.sort(key=len, reverse=True)
 
     # 1) Колонки
-    for prop in mapper.attrs:
-        if not isinstance(prop, ColumnProperty):
-            continue
-        if prop.key in exclude_column_names:
-            continue
+    if include_columns:
 
-        column = prop.columns[0]
-        column_type = _python_type_from_col(column)
-
-        # Аннотация типа
-        if column.nullable:
-            annotations[prop.key] = Optional[column_type]  # type: ignore[name-defined]
-        else:
-            annotations[prop.key] = column_type
-
-        # Дефолты, если нужно
-        if include_column_defaults:
-            default_value = None
-            if column.default is not None and getattr(column.default, "is_scalar", False):
-                default_value = column.default.arg
-            elif column.server_default is not None and getattr(column.server_default.arg, "text", None):
-                default_value = column.server_default.arg.text
-
-            if default_value is not None:
-                attrs[prop.key] = Field(default=default_value)
+        for column_attr in mapper.column_attrs:
+            if not isinstance(column_attr, ColumnProperty):
+                continue
+            if column_attr.key in exclude_column_names:
+                continue
+            mapped_type = _get_sqlalchemy_mapped_types(sqlalchemy_model=sqlalchemy_model)[column_attr.key]
+            annotations[column_attr.key] = _define_sqlalchemy_column_mapped_type(type_=mapped_type)
 
     # 2) Свойства (@property)
     if include_properties:
@@ -240,10 +219,11 @@ def pydantic_schema_from_sqlalchemy_model(
                     ):
                         renamed_property_name_to_type[new_property_name] = property_type
                     else:
-                        raise ValueError(
-                            f"Property name '{property_name}' after removing prefix "
-                            f"conflicts with existing name '{new_property_name}'"
-                        )
+                        if not skip_property_name_if_exists:
+                            raise ValueError(
+                                f"Property name '{property_name}' after removing prefix "
+                                f"conflicts with existing name '{new_property_name}'"
+                            )
             property_name_to_type.update(renamed_property_name_to_type)
 
         # добавляем (колонки в приоритете)
